import { defineStore } from 'pinia';
import { api, jApi } from 'boot/axios';

export const useUserStore = defineStore('user', {
    state: () => {
        const token =
            sessionStorage.getItem('vnToken') ||
            localStorage.getItem('vnToken');

        return {
            token,
            isGuest: false,
            user: null,
            supplantedUser: null
        };
    },

    getters: {
        loggedIn: state => state.token != null
    },

    actions: {
        async getToken() {
            this.token =
                sessionStorage.getItem('vnToken') ||
                localStorage.getItem('vnToken');
        },
        async login(user, password, remember) {
            const params = { user, password };
            const res = await api.post('Accounts/login', params);

            if (remember) {
                localStorage.setItem('vnToken', res.data.token);
            } else {
                sessionStorage.setItem('vnToken', res.data.token);
            }

            this.$patch({
                token: res.data.token,
                name: user
            });
        },
        async logout() {
            if (this.token != null) {
                try {
                    await api.post('Accounts/logout');
                } catch (e) {}
                localStorage.removeItem('vnToken');
                sessionStorage.removeItem('vnToken');
            }
            this.$reset();
        },

        async fetchUser(userType = 'user') {
            try {
                const userData = await jApi.getObject(
                    'SELECT id, nickname, name FROM account.myUser'
                );
                this.$patch({ [userType]: userData });
            } catch (error) {
                console.error('Error fetching user: ', error);
            }
        },

        async supplantUser(supplantUser) {
            const json = await jApi.send('client/supplant', {
                supplantUser
            });
            this.token = json;
            sessionStorage.setItem('supplantUser', supplantUser);
            await this.fetchUser('supplantedUser');
        },

        async supplantInit() {
            const user = sessionStorage.getItem('supplantUser');
            if (user == null) return;
            await this.supplantUser(user);
        },

        async logoutSupplantedUser() {
            sessionStorage.removeItem('supplantUser');
            this.supplantedUser = null;
            await api.post('Accounts/logout');
            this.getToken();
            await this.fetchUser();
        }
    }
});
